@@ -92,28 +92,12 @@ struct nxt_tls_init_s {
 };
 
 
-struct nxt_tls_ticket_s {
-    uint8_t                       aes128;
-    u_char                        name[16];
-    u_char                        hmac_key[32];
-    u_char                        aes_key[32];
-};
-
-
-struct nxt_tls_tickets_s {
-    nxt_uint_t                    count;
-    nxt_tls_ticket_t              tickets[];
-};
-
-
 #if (NXT_HAVE_OPENSSL)
 extern const nxt_tls_lib_t        nxt_openssl_lib;
 
 void nxt_cdecl nxt_openssl_log_error(nxt_task_t *task, nxt_uint_t level,
     const char *fmt, ...);
 u_char *nxt_openssl_copy_error(u_char *p, u_char *end);
-nxt_int_t nxt_openssl_base64_decode(u_char *d, size_t dlen, const u_char *s,
-    size_t slen);
 #endif
 
 #if (NXT_HAVE_GNUTLS)
