@@ -60,9 +60,14 @@ static void nxt_python_request_handler(nxt_unit_request_info_t *req);
 
 static PyObject *nxt_python_create_environ(nxt_python_app_conf_t *c);
 static PyObject *nxt_python_copy_environ(nxt_unit_request_info_t *req);
-static PyObject *nxt_python_get_environ(nxt_python_ctx_t *pctx);
+static PyObject *nxt_python_get_environ(nxt_python_ctx_t *pctx,
+    nxt_python_target_t *app_target);
 static int nxt_python_add_sptr(nxt_python_ctx_t *pctx, PyObject *name,
     nxt_unit_sptr_t *sptr, uint32_t size);
+static int nxt_python_add_char(nxt_python_ctx_t *pctx, PyObject *name,
+    char *src, uint32_t size);
+static int nxt_python_add_py_string(nxt_python_ctx_t *pctx, PyObject *name,
+    PyObject *value);
 static int nxt_python_add_field(nxt_python_ctx_t *pctx,
     nxt_unit_field_t *field, int n, uint32_t vl);
 static PyObject *nxt_python_field_name(const char *name, uint8_t len);
@@ -137,6 +142,7 @@ static PyObject  *nxt_py_query_string_str;
 static PyObject  *nxt_py_remote_addr_str;
 static PyObject  *nxt_py_request_method_str;
 static PyObject  *nxt_py_request_uri_str;
+static PyObject  *nxt_py_script_name_str;
 static PyObject  *nxt_py_server_addr_str;
 static PyObject  *nxt_py_server_name_str;
 static PyObject  *nxt_py_server_port_str;
@@ -156,6 +162,7 @@ static nxt_python_string_t nxt_python_strings[] = {
     { nxt_string("REMOTE_ADDR"), &nxt_py_remote_addr_str },
     { nxt_string("REQUEST_METHOD"), &nxt_py_request_method_str },
     { nxt_string("REQUEST_URI"), &nxt_py_request_uri_str },
+    { nxt_string("SCRIPT_NAME"), &nxt_py_script_name_str },
     { nxt_string("SERVER_ADDR"), &nxt_py_server_addr_str },
     { nxt_string("SERVER_NAME"), &nxt_py_server_name_str },
     { nxt_string("SERVER_PORT"), &nxt_py_server_port_str },
@@ -300,11 +307,12 @@ nxt_python_wsgi_done(void)
 static void
 nxt_python_request_handler(nxt_unit_request_info_t *req)
 {
-    int               rc;
-    PyObject          *environ, *args, *response, *iterator, *item;
-    PyObject          *close, *result, *application;
-    nxt_bool_t        prepare_environ;
-    nxt_python_ctx_t  *pctx;
+    int                  rc;
+    PyObject             *environ, *args, *response, *iterator, *item;
+    PyObject             *close, *result;
+    nxt_bool_t           prepare_environ;
+    nxt_python_ctx_t     *pctx;
+    nxt_python_target_t  *target;
 
     pctx = req->ctx->data;
 
@@ -327,7 +335,9 @@ nxt_python_request_handler(nxt_unit_request_info_t *req)
 
     prepare_environ = 1;
 
-    environ = nxt_python_get_environ(pctx);
+    target = &nxt_py_targets->target[req->request->app_target];
+
+    environ = nxt_python_get_environ(pctx, target);
     if (nxt_slow_path(environ == NULL)) {
         rc = NXT_UNIT_ERROR;
         goto done;
@@ -348,8 +358,7 @@ nxt_python_request_handler(nxt_unit_request_info_t *req)
     Py_INCREF(pctx->start_resp);
     PyTuple_SET_ITEM(args, 1, pctx->start_resp);
 
-    application = nxt_py_targets->target[req->request->app_target].application;
-    response = PyObject_CallObject(application, args);
+    response = PyObject_CallObject(target->application, args);
 
     Py_DECREF(args);
 
@@ -580,11 +589,14 @@ nxt_python_copy_environ(nxt_unit_request_info_t *req)
 
 
 static PyObject *
-nxt_python_get_environ(nxt_python_ctx_t *pctx)
+nxt_python_get_environ(nxt_python_ctx_t *pctx,
+    nxt_python_target_t *app_target)
 {
     int                 rc;
-    uint32_t            i, j, vl;
+    char                *path;
+    uint32_t            i, j, vl, path_length;
     PyObject            *environ;
+    nxt_str_t           prefix;
     nxt_unit_field_t    *f, *f2;
     nxt_unit_request_t  *r;
 
@@ -604,13 +616,28 @@ nxt_python_get_environ(nxt_python_ctx_t *pctx)
                            r->target_length));
     RC(nxt_python_add_sptr(pctx, nxt_py_query_string_str, &r->query,
                            r->query_length));
-    RC(nxt_python_add_sptr(pctx, nxt_py_path_info_str, &r->path,
-                           r->path_length));
+
+    prefix = app_target->prefix;
+    path_length = r->path_length;
+    path = nxt_unit_sptr_get(&r->path);
+    if (prefix.length > 0
+        && ((path_length > prefix.length && path[prefix.length] == '/')
+            || path_length == prefix.length)
+        && memcmp(prefix.start, path, prefix.length) == 0)
+    {
+        RC(nxt_python_add_py_string(pctx, nxt_py_script_name_str,
+                                    app_target->py_prefix));
+
+        path += prefix.length;
+        path_length -= prefix.length;
+    }
+
+    RC(nxt_python_add_char(pctx, nxt_py_path_info_str, path, path_length));
 
     RC(nxt_python_add_sptr(pctx, nxt_py_remote_addr_str, &r->remote,
                            r->remote_length));
-    RC(nxt_python_add_sptr(pctx, nxt_py_server_addr_str, &r->local,
-                           r->local_length));
+    RC(nxt_python_add_sptr(pctx, nxt_py_server_addr_str, &r->local_addr,
+                           r->local_addr_length));
 
     if (r->tls) {
         RC(nxt_python_add_obj(pctx, nxt_py_wsgi_uri_scheme_str,
@@ -692,10 +719,16 @@ static int
 nxt_python_add_sptr(nxt_python_ctx_t *pctx, PyObject *name,
     nxt_unit_sptr_t *sptr, uint32_t size)
 {
-    char      *src;
-    PyObject  *value;
+    return nxt_python_add_char(pctx, name, nxt_unit_sptr_get(sptr), size);
+}
 
-    src = nxt_unit_sptr_get(sptr);
+
+static int
+nxt_python_add_char(nxt_python_ctx_t *pctx, PyObject *name,
+    char *src, uint32_t size)
+{
+    int       res;
+    PyObject  *value;
 
     value = PyString_FromStringAndSize(src, size);
     if (nxt_slow_path(value == NULL)) {
@@ -707,17 +740,25 @@ nxt_python_add_sptr(nxt_python_ctx_t *pctx, PyObject *name,
         return NXT_UNIT_ERROR;
     }
 
+    res = nxt_python_add_py_string(pctx, name, value);
+
+    Py_DECREF(value);
+
+    return res;
+}
+
+
+static int nxt_python_add_py_string(nxt_python_ctx_t *pctx, PyObject *name,
+    PyObject *value)
+{
     if (nxt_slow_path(PyDict_SetItem(pctx->environ, name, value) != 0)) {
         nxt_unit_req_error(pctx->req,
                            "Python failed to set the \"%s\" environ value",
                            PyUnicode_AsUTF8(name));
-        Py_DECREF(value);
 
         return NXT_UNIT_ERROR;
     }
 
-    Py_DECREF(value);
-
     return NXT_UNIT_OK;
 }
 
@@ -822,11 +863,38 @@ nxt_python_field_value(nxt_unit_field_t *f, int n, uint32_t vl)
     char      *p, *src;
     PyObject  *res;
 
+    src = nxt_unit_sptr_get(&f->value);
+
 #if PY_MAJOR_VERSION == 3
-    res = PyUnicode_New(vl, 255);
+    if (nxt_slow_path(n > 1)) {
+        char  *ptr;
+
+        p = nxt_unit_malloc(NULL, vl + 1);
+        if (nxt_slow_path(p == NULL)) {
+            return NULL;
+        }
+
+        ptr = p;
+        p = nxt_cpymem(p, src, f->value_length);
+
+        for (i = 1; i < n; i++) {
+            p = nxt_cpymem(p, ", ", 2);
+
+            src = nxt_unit_sptr_get(&f[i].value);
+            p = nxt_cpymem(p, src, f[i].value_length);
+        }
+        *p = '\0';
+
+        src = ptr;
+    }
+
+    res = PyUnicode_DecodeCharmap(src, vl, NULL, NULL);
+
+    if (nxt_slow_path(n > 1)) {
+        nxt_unit_free(NULL, src);
+    }
 #else
     res = PyString_FromStringAndSize(NULL, vl);
-#endif
 
     if (nxt_slow_path(res == NULL)) {
         return NULL;
@@ -834,7 +902,6 @@ nxt_python_field_value(nxt_unit_field_t *f, int n, uint32_t vl)
 
     p = PyString_AS_STRING(res);
 
-    src = nxt_unit_sptr_get(&f->value);
     p = nxt_cpymem(p, src, f->value_length);
 
     for (i = 1; i < n; i++) {
@@ -843,6 +910,7 @@ nxt_python_field_value(nxt_unit_field_t *f, int n, uint32_t vl)
         src = nxt_unit_sptr_get(&f[i].value);
         p = nxt_cpymem(p, src, f[i].value_length);
     }
+#endif
 
     return res;
 }
